<?php

use \Bitrix\Main\Web\HttpClient;
use Bitrix\Main\Data\Cache;
use Bitrix\Main\Loader;
use \Bitrix\Sale\Order as Order;
// Подключаем языковые файлы.
IncludeModuleLangFile(__FILE__);

/**
 * Class DellinAPI
 * Класс предназначен для расчета стоимости доставки.
 */
class DellinAPI
{


    /**
     * Id для авторизации в API деловых линий
     * @var string
     */
    protected static $dellinSessionId = false;

    /**
     * Получаем КЛАДР код по названию города.
     * Работает только с городами РФ.
     * @param integer $bx_location_to_id
     * @return string
     */
    public static function GetCityKLADRCode($bx_location_to_id,$altLocationName)
    {
        /*$kladr_code = '';
        $life_time = 24*60*60;
        $cache_id = 'DELLIN_LOCATION_KLADR_CODE|' . $bx_location_to_id;
        $cache = Cache::createInstance();
        if( $cache->initCache( $life_time, $cache_id)){
            $cache_data = $cache->GetVars();
            $kladr_code = $cache_data['VALUE'];
            echo "<pre>";
            var_dump($cache_data);
            echo "</pre>";
            die();
        }else{*/
            try{
                $db_vars = CSaleLocation::GetList(false, array("CODE" => $bx_location_to_id, "LID" => "ru"));
                $bx_location = $db_vars->Fetch();

                if(!$bx_location){
                    $db_vars = CSaleLocation::GetList(false, array("ID" => $bx_location_to_id, "LID" => "ru"));
                    $bx_location = $db_vars->Fetch();
                }
                // Формируем строку запроса.
                if($bx_location){

                    $dl_locations = ApiCore::SearchCity($bx_location['CITY_NAME']);
                }elseif($altLocationName){
                    $arName = explode(" ",trim($altLocationName));
                    if($arName > 3){
                        usort($arName,array('DellinApi','sort_func'));
                        $searchString = implode(" ",array_slice($arName,0,2));
                        $altLocationName = $searchString;
                    }
                    $dl_locations = ApiCore::SearchCity($altLocationName);
                    $bx_location['CITY_NAME'] = $altLocationName;
                }
                if (count($dl_locations) > 0) {
                    $dl_city = ApiCore::SelectCityByRegion($dl_locations, $bx_location['CITY_NAME']);
                    $kladr_code = (string)$dl_city->code;
                    /*$cache->startDataCache($life_time,$cache_id);
                    if($cache->isStarted()){
                        $cache->EndDataCache(
                            array(
                                "VALUE" => $kladr_code
                            )
                        );
                    }*/
                }else{
                    //$cache->abortDataCache();
                    return $dl_locations->errors;
                }
            }catch( SystemException $e ){
                ShowError( $e->getMessage() );
            }
        //}
        return $kladr_code;
    }

    /**
     * Рассчитываем кол-во грузовых мест.
     * @param array $arOrder
     * @param array $arConfig
     * @return int
     */
    protected static function GetNumbersOfCargoPlaces($arOrder, $arConfig)
    {
        $numbers_of_places = 1;

        switch ($arConfig['LOADING_GROUPING_OF_GOODS']['VALUE']) {
            // Если считаем весь заказ, как 1 грузоместо.
            case 'ONE_CARGO_SPACE':
                break;

            // Если группируем каждый вид товара, как отдельное грузоместо.
            case 'SEPARATED_CARGO_SPACE':
                $numbers_of_places = count($arOrder['ITEMS']);
                break;

            // Если каждая единица товара - отдельное грузоместо.
            case 'SINGLE_ITEM_SINGLE_SPACE':
                $numbers_of_places = 0;

                foreach ($arOrder['ITEMS'] as $item) {
                    $numbers_of_places += $item['QUANTITY'];
                }
                break;
        }
        return $numbers_of_places;
    }

    /**
     * Конвертируем граммы в килограммы.
     * @param float $bx_goods_weight_in_gram
     * @return bool|float|int
     */
    protected static function ConvertWeightFromGramToKilogram($bx_goods_weight_in_gram)
    {
        $weight_in_kg = CSaleMeasure::Convert((float)$bx_goods_weight_in_gram, "G", "KG");
        return $weight_in_kg > 0 ? $weight_in_kg : 0.01;
    }

    /**
     * Получаем массив с параметрами веса, основываясь на перегрузе
     * @param float $bx_compare_weight
     * @param null|float $bx_set_weight
     * @return array
     */
    public static function GetWeightArray($bx_compare_weight, $bx_set_weight = null) {
        $compare_weight = self::ConvertWeightFromGramToKilogram($bx_compare_weight);
        $total_weight = is_null($bx_set_weight) ? $compare_weight : self::ConvertWeightFromGramToKilogram($bx_set_weight);

        if (ApiCore::IsOversizedWeight($compare_weight)) {
            return array('sized' => $total_weight, 'oversized' => $total_weight);
        }

        return array('sized' => $total_weight);
    }

    /**
     * Определяем параметры "sizedWeight" и "oversizedWeight" по способу группировки груза
     * @param array $arOrder
     * @param array $arConfig
     * @return array
     */
    static function CalculateWeight($arOrder, $arConfig)
    {
        $arWeight = array();
        switch ($arConfig['LOADING_GROUPING_OF_GOODS']['VALUE']) {
            // Если считаем весь заказ, как 1 грузоместо.

            case 'ONE_CARGO_SPACE':
                $arWeight = self::GetWeightArray($arOrder['WEIGHT']);
                break;

            // Если группируем каждый вид товара, как отдельное грузоместо.
            case 'SEPARATED_CARGO_SPACE':
                $max_item_weight = 0;
                $sum_item_weight = 0;
                foreach ($arOrder['ITEMS'] as $item) {
                    $item_weight = $item['WEIGHT'] * $item['QUANTITY'];
                    $sum_item_weight += $item_weight;

                    if ($item_weight > $max_item_weight) {
                        $max_item_weight = $item_weight;
                    }
                }

                $arWeight = self::GetWeightArray($max_item_weight, $sum_item_weight);
                break;

            // Если каждая единица товара - отдельное грузоместо.
            case 'SINGLE_ITEM_SINGLE_SPACE':
                $max_item_weight = 0;
                $sum_item_weight = 0;

                foreach ($arOrder['ITEMS'] as $item) {
                    $sum_item_weight += $item['WEIGHT'] * $item['QUANTITY'];

                    if ($item['WEIGHT'] > $max_item_weight) {
                        $max_item_weight = $item['WEIGHT'];
                    }
                }
                $arWeight = self::GetWeightArray($max_item_weight, $sum_item_weight);
                break;
        }

        if(empty($arWeight)) $arWeight['sized'] = 0.1;
        return $arWeight;
    }

    /**
     * Определяем параметры "sizedVolume" и "oversizedVolume" по способу группировки груза
     * @param array $arOrder
     * @param array $arConfig
     * @return array
     */
    static function CalculateVolume($arOrder, $arConfig) {
        $arVolume = array();

        $totalVolume = 0;
        $totalOversizedVolume = 0;

        foreach ($arOrder['ITEMS'] as $item) {
            $bx_item_length = (int) $item['DIMENSIONS']['LENGTH']?:0;
            $bx_item_width = (int) $item['DIMENSIONS']['WIDTH']?:0;
            $bx_item_height = (int) $item['DIMENSIONS']['HEIGHT']?:0;

            $totalVolume += $item['QUANTITY'] * $bx_item_length * $bx_item_width * $bx_item_height;
        }
        $totalVolume = ApiCore::ConvertVolumeFromCubicMillimeterToCubicMeter($totalVolume);
        switch ($arConfig['LOADING_GROUPING_OF_GOODS']['VALUE']) {
            // Если считаем весь заказ, как 1 грузоместо.
            case 'ONE_CARGO_SPACE':
                $bx_item_max_size = array('length' => 0, 'width' => 0, 'height' => 0);

                foreach ($arOrder['ITEMS'] as $item) {
                    $bx_item_length = $item['DIMENSIONS']['LENGTH'];
                    $bx_item_width = $item['DIMENSIONS']['WIDTH'];
                    $bx_item_height = $item['DIMENSIONS']['HEIGHT'];

                    if ($bx_item_length > $bx_item_max_size['length']) {
                        $bx_item_max_size['length'] = $bx_item_length;
                    }

                    if ($bx_item_width > $bx_item_max_size['width']) {
                        $bx_item_max_size['width'] = $bx_item_width;
                    }

                    if ($bx_item_height > $bx_item_max_size['height']) {
                        $bx_item_max_size['height'] = $bx_item_height;
                    }
                }

                $arVolume = ApiCore::GetVolumeArrayBySize($bx_item_max_size, $totalVolume, $totalVolume);
                break;

            // Если группируем каждый вид товара, как отдельное грузоместо.
            case 'SEPARATED_CARGO_SPACE':
                foreach ($arOrder['ITEMS'] as $item) {
                    $bx_item_length = $item['DIMENSIONS']['LENGTH'];
                    $bx_item_width = $item['DIMENSIONS']['WIDTH'];
                    $bx_item_height = $item['DIMENSIONS']['HEIGHT'];

                    $bx_item_volume = $item['QUANTITY'] * $bx_item_length * $bx_item_width * $bx_item_height;

                    if (ApiCore::IsOversizedVolume($bx_item_volume)) {
                        $totalOversizedVolume += $bx_item_volume;
                    }
                }

                $arVolume = ApiCore::GetVolumeArrayByVolume(
                    $totalOversizedVolume,
                    $totalVolume,
                    ApiCore::ConvertVolumeFromCubicMillimeterToCubicMeter($totalOversizedVolume)
                );

                break;

            // Если каждая единица товара - отдельное грузоместо.
            case 'SINGLE_ITEM_SINGLE_SPACE':
                foreach ($arOrder['ITEMS'] as $item) {
                    $bx_item_length = $item['DIMENSIONS']['LENGTH'];
                    $bx_item_width = $item['DIMENSIONS']['WIDTH'];
                    $bx_item_height = $item['DIMENSIONS']['HEIGHT'];

                    $bx_item_volume = $bx_item_length * $bx_item_width * $bx_item_height;

                    if (ApiCore::IsOversizedVolume($bx_item_volume)) {
                        $totalOversizedVolume += $bx_item_volume;
                    }
                }
                $arVolume = ApiCore::GetVolumeArrayByVolume(
                    $totalOversizedVolume,
                    $totalVolume,
                    ApiCore::ConvertVolumeFromCubicMillimeterToCubicMeter($totalOversizedVolume)
                );

                break;
        }
        if(empty($arVolume) || $arVolume['sized']<=0.001){
            $arVolume['sized'] = 0.001;
        }
        return $arVolume;
    }

    /**
     * Получаем значения для поля "{arrival-derival}Services"
     * @param array $bx_fields
     * @return array
     */
    protected static function GetAdditionalServices($bx_fields)
    {
        $value_list = array();

        foreach ($bx_fields as $item) {
            if ($item == 'NULL') {
                continue;
            }

            $value_list[] = $item;
        }
        return $value_list;
    }

    /**
     * Получаем массив из id типов упаковки.
     * @param array $bx_fields
     * @return array
     */
    public static function GetSelectedPackingTypesId($requestType,$bx_fields)
    {
        if($requestType == 'calculator'){
            $packagesArray = ApiCore::$packing_types_id;
        }elseif($requestType == 'request'){
            $packagesArray = ApiCore::$request_packages_id;
        }

        $value_list = array();

        foreach ($bx_fields as $key => $item) {
            if ($item['VALUE'] == 'N') {
                continue;
            }

            $value_list[] = $packagesArray[$key];
        }
        return $value_list;
    }

    /**
     * Получаем информацио о доставке на основании настроек модуля и содержимого заказа.
     * @param array $arOrder
     * @param array $arConfig
     * @return array
     */
    public static function GetDeliveryData($arOrder, $arConfig)
    {
        if($_REQUEST['order']){
            $altLocationPropId = CSaleOrderProps::GetList(array(),array('TYPE'=>'LOCATION','PERSON_TYPE_ID'=>$arOrder['PERSON_TYPE_ID']),false,false,array())->Fetch()['INPUT_FIELD_LOCATION'];//ID инпута альтернативного ввода города (в настройках свойства типа location)
            $altLocationName = $_REQUEST['order']['ORDER_PROP_'.$altLocationPropId];//кастомное название населенного пункта в поле для альтернативного ввода местоположения
        }elseif($arOrder['ID']){
            $orderEx = DellinDelivery::GetOrder($arOrder['ID']);
            $altLocationName = $orderEx['ALT_LOCATION_NAME'];
            $arrivalKladr= CSaleOrderPropsValue::GetList(array(),array('ORDER_ID'=>$arOrder['ID'],'CODE'=>'STREET_KLADR'))->Fetch()['VALUE'];
        }
        //вход в систему для получения цен с учетом систем лояльности
        if($arConfig['LOGIN']['VALUE'] && $arConfig['PASSWORD']['VALUE']){
            $sessionId = self::GetDellinSessionId($arConfig);
        }

        // Получаем КЛАДР код населенного пункта, из которого нужно доставить груз.
        $delivery_from_kladr_code = (string)$arConfig['KLADR_CODE_DELIVERY_FROM']['VALUE'];
        // Получаем КЛАДР код населенного пункта, в который нужно доставить груз.
        if($arOrder['ID'] && isset($arrivalKladr)){
            $delivery_to_kladr_code = $arrivalKladr;
        }else{
            $delivery_to_kladr_code = self::GetCityKLADRCode($arOrder['LOCATION_TO'],$altLocationName);
        }
        // Получаем кол-во грузовых мест.

        $dl_api_numbers_of_cargo_places = self::GetNumbersOfCargoPlaces($arOrder, $arConfig);

        // Получаем вес.
        $dl_api_weight = self::CalculateWeight($arOrder, $arConfig);


        // Получаем объем.
        $dl_api_volume = self::CalculateVolume($arOrder, $arConfig);


        // Формируем данные для отправки.
        // Полный список параметров можно найти тут:
        // http://dev.dellin.ru/api/public/calculator/
        $dl_api_data = array(
            // Ключ API (обязательное поле).
            // string
            'appKey' => (string)$arConfig['API_KEY']['VALUE'],

            // Код КЛАДР пункта отправки (обязательное поле).
            // string
            'derivalPoint' => $delivery_from_kladr_code,

            // Необходима доставка груза от адреса.
            // boolean

            'derivalDoor' => $arConfig['IS_GOODS_LOADING']['VALUE'] == 'Y',

            // Требуются дополнительные услуги для доставки груза от адреса - боковая погрузка
            // array
            "derivalServices" => array(),

            // Код КЛАДР пункта прибытия (обязательное поле).
            // string
            'arrivalPoint' => $delivery_to_kladr_code,

            // Необходима доставка груза до адреса.
            // boolean
            'arrivalDoor' => $arConfig['IS_GOODS_UNLOADING']['VALUE'] == 'Y',

            // Требуются дополнительные услуги для доставки груза до адреса - боковая погрузка
            // array
            "arrivalServices" => array(),

            // Общий объём груза в кубических метрах (обязательный поле).
            // string

            'sizedVolume' =>  (string) round($dl_api_volume['sized'],3,PHP_ROUND_HALF_EVEN),

            // Общий вес груза в килограммах (обязательный поле).
            // string
            'sizedWeight' => (string) round($dl_api_weight['sized'],3,PHP_ROUND_HALF_EVEN),

            // Заявленная стоимость груза в рублях. При отсутсвии - груз не страхуется, при передаче 0 - страхуется без объявленной стоимости,
            // при передаче значения больше 0 - страхуется на указанную сумму.
            // float | integer
            'statedValue' => (string) round($arConfig['IS_INSURANCE_GOODS_WITH_DECLARED_PRICE']['VALUE'] == 'Y' ? (float)$arOrder['PRICE'] : 0,3,PHP_ROUND_HALF_EVEN),

            // Упаковать груз в упаковку?
            // array
            'packages' => self::GetSelectedPackingTypesId('calculator',array(
                'box'=> $arConfig['PACKING_FOR_GOODS_BOX']['VALUE'],
                'hard' => $arConfig['PACKING_FOR_GOODS_HARD']['VALUE'],
                'additional' => $arConfig['PACKING_FOR_GOODS_ADDITIONAL']['VALUE'],
                'bubble' => $arConfig['PACKING_FOR_GOODS_BUBBLE']['VALUE'],
                'bag' => $arConfig['PACKING_FOR_GOODS_BAG']['VALUE'],
                'pallet' => $arConfig['PACKING_FOR_GOODS_PALLET']['VALUE'],
                'car_glass' =>$arConfig['PACKING_FOR_GOODS_CAR_GLASS']['VALUE'],
                'car_parts' => $arConfig['PACKING_FOR_GOODS_CAR_PARTS']['VALUE'],
                'complex_pallet' => $arConfig['PACKING_FOR_GOODS_COMPLEX_PALLET']['VALUE'],
                'complex_hard' => $arConfig['PACKING_FOR_GOODS_COMPLEX_HARD']['VALUE'],
            )),
            // Количество мест, по-умолчанию расчет производится для одного места.
            // integer
            'quantity' => $dl_api_numbers_of_cargo_places,
            'productInfo'=>array(
                'type'=> 4,
                'productType'=> 5,
                'info'=>array(
                    array(
                        'param'=> 'module version',
                        'value'=> 'beta'
                    )
                )
            )
        );


        if($arConfig['PACKING_FOR_GOODS_BAG']['VALUE'] == 'Y'){
            $dl_api_data['bags_count'] = 1;
        }
        if($arConfig['PACKING_FOR_GOODS_BOX']['VALUE'] == 'Y'){
            $dl_api_data['packages_count'] = 1;
        }
        if($arConfig['PACKING_FOR_GOODS_CAR_GLASS']['VALUE'] == 'Y'){
            $dl_api_data['protect_auto_glass_count'] = 1;
        }
        if($arConfig['PACKING_FOR_GOODS_CAR_PARTS']['VALUE'] == 'Y'){
            $dl_api_data['protect_auto_part_count'] = 1;
        }

        if($arConfig['COUNTERAGENT']['VALUE'] !== "NULL"){
            $dl_api_data['cauid'] = $arConfig['COUNTERAGENT']['VALUE'];
        }
        if($sessionId){
            //id сессии авторизации
            $dl_api_data['sessionId'] = $sessionId;
        }
        // Если есть перегруз или негаборит, то добавим эти параметры.
        if ($dl_api_volume['oversized'] || $dl_api_weight['oversized']) {
            $dl_api_data['oversizedVolume'] = $dl_api_volume['sized'];
            $dl_api_data['oversizedWeight'] = $dl_api_weight['sized'];
        }
        // Добавляем дополнительные услуги, если отмечена галочка, что необходима погрузка товара
        if ($dl_api_data['derivalDoor']) {
            $dl_api_data['derivalServices'] = self::GetAdditionalServices(array(
                $arConfig['LOADING_TYPE']['VALUE'],
                $arConfig['LOADING_TRANSPORT_REQUIREMENTS']['VALUE'],
                $arConfig['LOADING_ADDITIONAL_EQUIPMENT']['VALUE']
            ));
        }
        // Добавляем дополнительные услуги, если отмечена галочка, что необходима разгрузка товара
        if ($dl_api_data['arrivalDoor']) {
            $dl_api_data['arrivalServices'] = self::GetAdditionalServices(array(
                $arConfig['UNLOADING_TYPE']['VALUE'],
                $arConfig['UNLOADING_TRANSPORT_REQUIREMENTS']['VALUE'],
                $arConfig['UNLOADING_ADDITIONAL_EQUIPMENT']['VALUE']
            ));
        }
        return $dl_api_data;

    }

    /**
     * Проверяем код на валидность.
     * @param array $data
     * @param array $arOrder
     * @param array $arConfig
     * @return bool
     */
    protected static function IsDataValid($data, $arOrder, $arConfig)
    {
        return
            (
                $arOrder['PRICE'] > 0
            ) && (
                $data["appKey"] !== ''
            ) && (
                $data["derivalPoint"] !== '' &&
                $data["arrivalPoint"] !== ''
            ) && (
                $arConfig['INTERCITY_HIDE']['VALUE'] == "N" || ($arConfig['INTERCITY_HIDE']['VALUE'] == "Y" && ($data['derivalPoint'] != $data['arrivalPoint'])) || ($arConfig['IS_GOODS_UNLOADING']['VALUE']=="Y" && $arConfig['IS_GOODS_LOADING']['VALUE'] == "Y")
            );
    }

    /**
     * Формируем строку ошибки.
     * @param $errors
     * @return string
     */
    protected static function GetResponseErrors($errors)
    {
        $errors_srt = implode('; ', $errors->messages);
        return $errors_srt;
    }

    /**
     * Расчет конечной стоимости.
     * @param object $response
     * @param array $arConfig
     * @return bool|float
     */
    protected static function CalculatePrice($response, $arConfig)
    {
        $total_price = false;
        $is_small_goods_price = $arConfig['IS_SMALL_GOODS_PRICE']['VALUE'] == 'Y';
        //$is_try_small_goods_price_calculate = $arConfig['TRY_SMALL_GOODS_PRICE_CALCULATE'] == 'Y';
        if ($response->price > 0) {
            $total_price = $response->price;
        }
        if ($is_small_goods_price && isset($response->small) && $response->small->price > 0 && $arConfig['IS_GOODS_LOADING']['VALUE']=='Y' && $arConfig['IS_GOODS_UNLOADING']['VALUE']=='Y') {
            $total_price = $response->small->price + $response->small->insurance + $response->small->notify->price;

            if(isset($response->packages)){
                $packages = array_diff_key(get_object_vars($response->packages), array("discount"=>"","discountDetails"=>"","premium"=>"","premiumDetails"=>"","required"=>""));
                $packagesSum = 0;
                foreach($packages as $package){
                    $packagesSum += $package;
                }
                $total_price += $packagesSum;
            }
        }
        /**/
        return $total_price;
    }

    /**
     * Расчет стоимости доставки.
     * @param array $arOrder
     * @param array $arConfig
     * @return array|bool
     */
    public static function  Calculate($arOrder, $arConfig)
    {
       // $result = array('STATUS' => 'ERROR', 'BODY' => GetMessage('DELLIN_CALCULATE_ERROR'));
        $data = self::GetDeliveryData($arOrder, $arConfig);
        //если выбрана настройка "скрывать этот способ доставки из города магазина, arrival и derival совпадают и не выбрана доставка от адреса до адреса, не производим расчет
        if (self::IsDataValid($data, $arOrder, $arConfig)) {
            $cache = new CPHPCache();
            $life_time = 10*60;
            $cache_id = 'DELLIN_CALCULATE|' . serialize($data) . '&' . serialize($arConfig);
            if ($cache->InitCache($life_time, $cache_id) && ($cache->GetVars()['VALUE'] !== null && $cache->GetVars()['VALUE']['BODY'][0] !== false )) {
                $cache_data = $cache->GetVars();
                $result = $cache_data['VALUE'];
                $_SESSION['current_terminals'] = $cache_data['TERMINALS'];
            } else {
                $http_client = new HttpClient();
                $http_client->setHeader('Content-Type', 'application/json', true);
                $response = ApiCore::sendApiRequest('calculator',$data);
                /* if($response == null){
                   echo "<pre>";
                   var_dump($response);
                   echo "</pre>";
                   die();
                }*/
                if (isset($response->errors)) {
                    $result['BODY'] = self::GetResponseErrors($response->errors);
                    if ($response->errors->messages && self::GetResponseErrors($response->errors) == null) {
                        $result['BODY'] = $response->errors->messages;
                    }else{
                        foreach($response->errors as $index => $error){
                            $result['STATUS'] = "ERROR";
                            $result['BODY'] .= $index.": ".$error->error." | ";
                        }
                    }
                } else {
                    unset($_SESSION['current_terminals']);
                    $_SESSION['current_terminals'] = $response->arrival->terminals;
                    if ($price = self::CalculatePrice($response, $arConfig)) {
                        $result['STATUS'] = 'OK';
                        $result['BODY'] = array($price, $response->time->nominative);
                    }
                    if(isset($response->small) && $arConfig['IS_SMALL_GOODS_PRICE']['VALUE']=="Y" && $arConfig['IS_GOODS_LOADING']['VALUE']=='Y' && $arConfig['IS_GOODS_UNLOADING']['VALUE']=='Y'){
                        $result['DELIVERY_TYPE'] = "small";
                    }else{
                        $result['DELIVERY_TYPE'] = "auto";
                    }
                    if ($arOrder['ID'] !== null) {
                        $arOrder = DellinDelivery::GetOrder($arOrder['ID']);
                        $config = DellinDelivery::GetDeliveryConfig($arOrder['CURRENT_DELIVERY_ID']);
                    }

                    $arVolume = DellinAPI::CalculateVolume($arOrder, $config);
                    $arWeight = DellinAPI::CalculateWeight($arOrder, $config);
                    $deliveryDelay = $config['DELIVERY_DELAY']['VALUE']? $config['DELIVERY_DELAY']['VALUE']: 0;
                    $produceData = array(
                        "appKey" => $config['API_KEY']['VALUE'],
                        "produceDate" => date("Y-m-d", mktime(0, 0, 0, date("m"), date("d") + $deliveryDelay, date("Y"))), //ориентировочная минимальная дата (сегодня + дней из настроек)
                        "sizedVolume" => $arVolume['sized'],
                        "sizedWeight" => $arWeight['sized'],
                        "derivalPoint" => $config['KLADR_CODE_DELIVERY_FROM']['VALUE'],
                        "derivalDoor" => ($config['IS_GOODS_LOADING']['VALUE'] == "Y") ? true : false,
                        "deliveryType" => "1",
                        "derivalPeriodVisit" => array(
                            "workStart" => ($config['WORK_START']['VALUE'] < 10?"0":"").$config['WORK_START']['VALUE'].":".(($config['WORK_START']['VALUE'] ==24)?"59":"00"),
                            "workEnd"   => ($config['WORK_END']['VALUE'] < 10?"0":"").$config['WORK_END']['VALUE'].":".(($config['WORK_END']['VALUE'] ==24)?"59":"00")
                        ),
                        "oversizedWeight" => $arWeight['oversized'],
                        "oversizedVolume" => $arVolume['oversized'],
                        "length" => (ApiCore::convertMMtoM($arOrder['MAX_LENGTH']) > 0) ? ApiCore::convertMMtoM($arOrder['MAX_LENGTH']) : 0.01,
                        "width" => (self::ConvertWeightFromGramToKilogram($arOrder['MAX_WIDTH']) > 0) ? self::ConvertWeightFromGramToKilogram($arOrder['MAX_WIDTH']) : 0.01,
                        "height" => (ApiCore::convertMMtoM($arOrder['MAX_HEIGHT']) > 0) ? ApiCore::convertMMtoM($arOrder['MAX_HEIGHT']) : 0.01
                    );
                    /*echo "<pre>";
                    var_dump(json_encode($produceData));
                    echo "</pre>";
                    die();*/
                    $produceDate = DellinDelivery::GetProduceDate($produceData, $config);

                    $nearestDate = $produceDate->nearestDate;
                    $result['STATUS'] = 'OK';
                    $result['BODY'] = array($price, $response->time->nominative, $nearestDate);
                    if ($produceDate->changeTime) {
                        $result['BODY'][] = $produceDate->changeTime;
                    }
                    if ($produceDate->DATE_CHANGED) {
                        $result['DATE_CHANGED'] = $produceDate->DATE_CHANGED;
                    }
                    $cache->StartDataCache($life_time, $cache_id);
                    $cache->EndDataCache(array('VALUE' => $result, 'TERMINALS' => $response->arrival->terminals));
                }

            }
        }else{
            return false;
        }
        if( $result['BODY'][0] == false) $result['STATUS'] = 'ERROR';
        return $result;
    }


    /**
     * Получаем sessionId из кэша или авторизуемся
     * */
    public function GetDellinSessionId($arConfig){
        if($arConfig['LOGIN']['VALUE'] && $arConfig['PASSWORD']['VALUE']){
            $hash = hash('md5',$arConfig['LOGIN']['VALUE'].$arConfig['PASSWORD']['VALUE']);
            $life_time = 30*24*60*60;
            $cache_id = $hash;
            $cache_path = 'dellinSessionId';
            $cache = new CPHPCache();
            if( $cache->initCache( $life_time, $cache_id,$cache_path) && !isset($arConfig['RESET_SESSION'])){
                $dellinSessionId = $cache->getVars()[$cache_id];
            }else{
                try{
                    $arParams['appkey'] = $_REQUEST['API_KEY']?trim($_REQUEST['API_KEY']):$arConfig['API_KEY']['VALUE'];
                    $arParams['login'] = $arConfig['LOGIN']['VALUE'];
                    $arParams['password'] = $arConfig['PASSWORD']['VALUE'];
                    $dellinSession = ApiCore::sendApiRequest('login',$arParams);
                    if( $dellinSession->sessionID !== Null &&  $dellinSession->sessionID !== false){
                        $cache->startDataCache($life_time,$cache_id,$cache_path);
                        $cache->EndDataCache(
                            array(
                                $cache_id =>  $dellinSession->sessionID
                            )
                        );
                        $dellinSessionId =  $dellinSession->sessionID;
                    }else{
                        return $dellinSession;
                    }
                }catch( SystemException $e ){
                    ShowError( $e->getMessage() );
                }

            }
            return $dellinSessionId;
        }else{
            $cache = new CPHPCache();
            $cache->CleanDir('dellinSessionId');
            return false;
        }

    }

    private static function sort_func($a,$b)
    {
        if (strlen($a) == strlen($b))
        {
            //  return 0;
        }

        return (strlen($a) > strlen( $b)) ? -1 : 1;
    }
}
