<?php
use \Bitrix\Sale\Delivery\Services\Manager as DeliveryManager;
use \Bitrix\Sale\Delivery\Services\Automatic as Automatic;
use \Bitrix\Sale\Order as Order;
use \Bitrix\Main\Data\Cache as Cache;
// Подключаем языковые файлы.
IncludeModuleLangFile(__FILE__);

/**
 * Class DellinDelivery
 * Определяет работу модуля доставки.
 * Класс реализован по документации 1С-Битрикс.
 * https://dev.1c-bitrix.ru/api_help/sale/delivery.php
 */
Class DellinDelivery
{
    protected static $deliveryConfig = array();
    protected static $moduleIncludePath;
    /**
     * Описание обработчика.
     * @return array
     */
    function Init()
    {
        self::$moduleIncludePath = $_SERVER['DOCUMENT_ROOT'].'/bitrix/modules/dellindev.delivery/include.php';
        return array(
            // Уникальный строковой идентификатор обработчика.
            'SID' => 'dellindev.delivery',

            // Название обработчика.
            'NAME' => GetMessage('DELLIN_NAME'),

            // Текстовое описание обработчика.
            'DESCRIPTION' => GetMessage('DELLIN_DESCRIPTION'),

            // Внутреннее описание обработчика, отображаемое при конфигурации обработчика в Панели Управления.
            'DESCRIPTION_INNER' => GetMessage('DELLIN_DESCRIPTION_INNER'),

            // Идентификатор базовой валюты обработчика.
            'BASE_CURRENCY' => 'RUR',

            // Путь к файлу обработчика. Нужен для корректного автоматического копирования обработчика (ещё не реализовано).
            // В подавляющем большинстве случаев достаточно значения '__FILE__'.
            'HANDLER' => __FILE__,

            // Название метода, возвращающего массив настроек валидатора.
            // В случае реализации обработчика в виде класса, значение представляет собой массив ('имя_класса', 'имя_метода').
            'GETCONFIG' => array('DellinDelivery', 'GetConfig'),

            //Название метода, возвращающего значения настроек
            // В случае реализации обработчика в виде класса, значение представляет собой массив ('имя_класса', 'имя_метода').
            'GETCONFIGVALUES' => array('DellinDelivery', 'GetConfigValues'),

            // Название метода, отвечающего за проверку настроек обработчика и преобразование массива настроек в строку для сохранения.
            // В случае реализации обработчика в виде класса, значение представляет собой массив ('имя_класса', 'имя_метода').
            // В случае отсутствия этого метода массив настроек будет сохранен в базу в сериализованном виде.
            'DBSETSETTINGS' => array('DellinDelivery', 'SetSettings'),

            // Название метода, отвечающего за обратное преобразование строки настроек обработчика в массив.
            // В случае реализации обработчика в виде класса, значение представляет собой массив ('имя_класса', 'имя_метода').
            'DBGETSETTINGS' => array('DellinDelivery', 'GetSettings'),

            // Название метода, отвечающего за дополнительную проверку совместимости профилей обработки с параметрами заказа.
            // Если метод отсутствует, дополнительная проверка не будет проводиться. В случае реализации обработчика в виде класса,
            // значение представляет собой массив ('имя_класса', 'имя_метода').
            'COMPABILITY' => array('DellinDelivery', 'Compability'),

            // Название метода, осуществляющего расчёт стоимости доставки. В случае реализации обработчика в виде класса,
            // значение представляет собой массив ('имя_класса', 'имя_метода').
            'CALCULATOR' => array('DellinDelivery', 'Calculate'),



            // Массив профилей обработки.
            'PROFILES' => array(
                // строковой_идентификато_профиля.
                'dellin_delivery_default' => array(
                    // Название профиля.
                    'TITLE' => GetMessage('DELLIN_TITLE'),

                    // Описание профиля.
                    'DESCRIPTION' => GetMessage('DELLIN_TITLE_DESCRIPTION'),

                    // Веса указываются в граммах (минимальный_вес, максимальный_вес).
                    'RESTRICTIONS_WEIGHT' => array(0),

                    // Суммы указываются в базовой валюте обработчика (минимальная_сумма_заказа, максимальная_сумма_заказа).
                    'RESTRICTIONS_SUM' => array(0)
                )
            )
        );
    }

    /**
     * Запрос конфигурации службы доставки.
     * @return array
     */
    function GetConfig()
    {
        global $APPLICATION;
        if($APPLICATION->GetCurPage() == "/bitrix/admin/sale_delivery_service_edit.php"){
            /*$opfList = ApiCore::GetOpfList()['list'];
            $arOPF = array("NULL"=>GetMessage('DELLIN_SETTINGS_VALUES_SENDER_FORM_NO'));
            $arOpfValues = array();
            $arCountries = ApiCore::GetCountries();
            foreach($arCountries as $country){
                $arCountriesValues[$country['countryUID']] = $country['country'];
            }

            foreach($opfList as $opf){
                $arOpfValues[$opf['uid']]="(".$arCountriesValues[$opf['countryUID']].") ".$opf['name'];
            }

            natcasesort($arOpfValues);
            $arOPF = array_merge($arOPF,$arOpfValues);*/
            $oldConfig = self::GetDeliveryConfig($_REQUEST['ID']);
            $sessionId = DellinAPI::GetDellinSessionId($oldConfig);

            $arTerminalIdValues = array('NULL' => ($oldConfig['KLADR_CODE_DELIVERY_FROM']['VALUE']) ? GetMessage('DELLIN_SETTINGS_VALUES_TERMINAL_ID_NO') : GetMessage('DELLIN_SETTINGS_KLADR_CODE_ERROR'));
            if ($oldConfig['KLADR_CODE_DELIVERY_FROM']['VALUE']) {
                $data = array(
                    "appkey" => $oldConfig['API_KEY']['VALUE'],
                    "code" => $oldConfig['KLADR_CODE_DELIVERY_FROM']['VALUE'],
                    "direction" => "derival"
                );
                if($sessionId){
                    $data["sessionID"] = $sessionId;
                }
                $terminalsOb = ApiCore::sendApiRequest('requestTerminal',$data);
                foreach ($terminalsOb->terminals as $terminal) {
                    $terminalInfo = ApiCore::GetTerminals($terminal->city_code,$terminal->id);
                    if($terminalInfo->receiveCargo){ //если терминал может принять товар
                        $arTerminalIdValues[$terminal->id] = $terminal->address;
                    }
                }
            }
        }
        $arTimeValuesBreak[0] = "Нет";
        for($n = 0;$n<=24;$n++){
            if($n == 24){
                $arTimeValues[]= "23:59";
                $arTimeValuesBreak[]= "23:59";
            }else{
                $arTimeValues[]= (($n<10)?"0":"").$n.":00";
                $arTimeValuesBreak[]= (($n<10)?"0":"").$n.":00";
            }

        }

        $arConfig = array(
            'CONFIG_GROUPS' => array(
                'dellin_delivery_settings' => GetMessage('DELLIN_SETTINGS'),
            ),

            'CONFIG' => array(
                // Настройки API.

                'HEADER_API_SETTINGS' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_API_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                'PATH_TO_ORDER' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_PATH_TO_ORDER'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '50'
                ),
                'API_KEY' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_API_KEY'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '50'
                ),
                'LOGIN'=>array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_LOGIN'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'PASSWORD'=>array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PASSWORD'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),

                'REQUEST_EMAIL' => array(
                    'REQUIRED'   => 'Y',
                    'TYPE'      => 'STRING',
                    'TITLE'     => GetMessage('DELLIN_SETTINGS_TITLE_REQUEST_EMAIL'),
                    'DEFAULT'   => '',
                    'GROUP'     => 'dellin_delivery_settings',
                    'SIZE'      => '100'
                ),

                'COUNTERAGENT'=>array(
                    'TYPE'=>'STRING',
                    'DEFAULT'=>'',
                    'TITLE'=>GetMessage('DELLIN_SETTINGS_VALUES_COUNTERAGENT_TITLE'),
                    'GROUP'=>'dellin_delivery_settings',
                ),

                'HEADER_DELIVERY_DELAY' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_DELIVERY_DELAY_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                'DELIVERY_DELAY'=>array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_DELIVERY_DELAY'),
                    'DEFAULT' => '1',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                /*'DELIVERY_DELAY_TO_PRODUCE_DATE'=>array(
                    'TYPE' => 'CHECKBOX',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_DELIVERY_DELAY_TO_PRODUCE_DATE'),
                    'DEFAULT' => 'N',
                    'GROUP' => 'dellin_delivery_settings',
                ),*/


                // Пункт отправки.
                'HEADER_DELIVERY_FROM' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_DELIVERY_FROM_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                'KLADR_CODE_DELIVERY_FROM' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_KLADR_CODE_DELIVERY_FROM'),
                    'DEFAULT' => '7800000000000000000000000',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '30'
                ),
                // Отправитель.
                'HEADER_SENDER' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

                'OPF_COUNTRY'=> array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_OPF_COUNTRY'),
                    'DEFAULT' => 'NULL',
                    'GROUP' => 'dellin_delivery_settings',
                ),

                'SENDER_FORM'=> array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_FORM'),
                    'DEFAULT' => 'NULL',
                    'GROUP' => 'dellin_delivery_settings',
                    //'VALUES'=> $arOPF
                ),
                'SENDER_NAME' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_NAME'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_INN' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_INN'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_CONTACT_PERSON_NAME' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_CONTACT_PERSON_NAME'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_CONTACT_PHONE' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_CONTACT_PHONE'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_CONTACT_EMAIL' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_CONTACT_EMAIL'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                //Параметры юридического адреса отправителя
                'HEADER_SENDER_JURIDICAL_ADDRESS' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_ADDRESS_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

                'SENDER_JURIDICAL_KLADR_CITY' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_KLADR_CITY'),
                    'DEFAULT' => '7800000000000000000000000',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '30'
                ),

                'SENDER_JURIDICAL_KLADR_STREET' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_KLADR_STREET'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_JURIDICAL_HOUSE' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_HOUSE'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_JURIDICAL_BUILDING' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_BUILDING'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_JURIDICAL_STRUCTURE' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_STRUCTURE'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                'SENDER_JURIDICAL_FLAT' => array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_JURIDICAL_FLAT'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),
                /*'SENDER_STREET_KLADR'=>array(
                    'TYPE' => 'STRING',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SENDER_STREET_KLADR'),
                    'DEFAULT' => '',
                    'GROUP' => 'dellin_delivery_settings',
                    'SIZE' => '100'
                ),*/

                // Малогабаритный груз.
                'HEADER_SMALL_GOODS' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_SMALL_GOODS_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'IS_SMALL_GOODS_PRICE' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_IS_SMALL_GOODS_PRICE'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                /*'TRY_SMALL_GOODS_PRICE_CALCULATE' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_TRY_SMALL_GOODS_PRICE_CALCULATE'),
                    'GROUP' => 'dellin_delivery_settings'
                ),*/

                // Страхование.
                'HEADER_INSURANCE' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_INSURANCE_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                'IS_INSURANCE_GOODS_WITH_DECLARED_PRICE' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_IS_INSURANCE_GOODS_WITH_DECLARED_PRICE'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

                // Погрузка. Т. е. груз загружается у отправителя.
                'HEADER_GOODS_LOADING' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_GOODS_LOADING_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                ),

                'HEADER_WORK_TIME_RESTRICTION' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_WORK_TIME_RESTRICTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'WORK_START' =>  array(
                    'TYPE'=>'DROPDOWN',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_WORK_START_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => $arTimeValues
                ),
                'WORK_END' => array(
                    'TYPE' => 'DROPDOWN',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_WORK_END_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => $arTimeValues
                ),
                'WORK_BREAK_START' =>  array(
                    'TYPE' => 'DROPDOWN',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_WORK_BREAK_START_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => $arTimeValuesBreak
                ),
                'WORK_BREAK_END' =>  array(
                    'TYPE' => 'DROPDOWN',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_WORK_BREAK_END_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => $arTimeValuesBreak
                ),

                'TERMINAL_ID'=>array(
                    'TYPE'=>'STRING',
                    'DEFAULT'=>'',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_VALUES_TERMINAL_ID_TITLE'),
                    'GROUP'=>'dellin_delivery_settings'
                ),

                'IS_GOODS_LOADING' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_IS_GOODS_LOADING'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

                'LOADING_ADDRESS'   =>  array(
                    'TYPE' => 'STRING',
                    'DEFAULT' => '',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_LOADING_ADDRESS'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                'LOADING_HOUSE_NUM' =>  array(
                    'TYPE' => 'STRING',
                    'DEFAULT' => '',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_LOADING_HOUSE'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

                'LOADING_TYPE' => array(
                    'TYPE' => 'DROPDOWN',
                    'DEFAULT' => 'NULL',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_LOADING_TYPE'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'NULL' => GetMessage('DELLIN_SETTINGS_VALUES_LOADING_TYPE_BACK'),
                        '0xb83b7589658a3851440a853325d1bf69' => GetMessage('DELLIN_SETTINGS_VALUES_LOADING_TYPE_SIDE'),
                        '0xabb9c63c596b08f94c3664c930e77778' => GetMessage('DELLIN_SETTINGS_VALUES_LOADING_TYPE_UP'),
                    ),
                ),
                'LOADING_TRANSPORT_REQUIREMENTS' => array(
                    'TYPE' => 'DROPDOWN',
                    'DEFAULT' => 'NULL',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_TRANSPORT_REQUIREMENTS'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'NULL' => GetMessage('DELLIN_SETTINGS_VALUES_TRANSPORT_REQUIREMENTS_NO'),
                        '0x9951e0ff97188f6b4b1b153dfde3cfec' => GetMessage('DELLIN_SETTINGS_VALUES_TRANSPORT_REQUIREMENTS_OPEN'),
                        '0x818e8ff1eda1abc349318a478659af08' => GetMessage('DELLIN_SETTINGS_VALUES_TRANSPORT_REQUIREMENTS_TENT')
                    ),
                ),
                'LOADING_ADDITIONAL_EQUIPMENT' => array(
                    'TYPE' => 'DROPDOWN',
                    'DEFAULT' => 'NULL',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_ADDITIONAL_EQUIPMENT'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'NULL' => GetMessage('DELLIN_SETTINGS_VALUES_ADDITIONAL_EQUIPMENT_NO'),
                        '0x92fce2284f000b0241dad7c2e88b1655' => GetMessage('DELLIN_SETTINGS_VALUES_ADDITIONAL_EQUIPMENT_TAIL_LIFT'),
                        '0x88f93a2c37f106d94ff9f7ada8efe886' => GetMessage('DELLIN_SETTINGS_VALUES_ADDITIONAL_EQUIPMENT_MANIPULATOR'),
                    ),
                ),
                'LOADING_GROUPING_OF_GOODS' => array(
                    'TYPE' => 'RADIO',
                    'DEFAULT' => 'ONE_CARGO_SPACE',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_GROUPING_OF_GOODS'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'ONE_CARGO_SPACE' => GetMessage('DELLIN_SETTINGS_VALUES_GROUPING_OF_GOODS_ONE_CARGO_SPACE'),
                        'SEPARATED_CARGO_SPACE' => GetMessage('DELLIN_SETTINGS_VALUES_GROUPING_OF_GOODS_SEPARATED_CARGO_SPACE'),
                        'SINGLE_ITEM_SINGLE_SPACE' => GetMessage('DELLIN_SETTINGS_VALUES_GROUPING_OF_GOODS_SINGLE_ITEM_SINGLE_SPACE'),
                    ),
                ),

                // Выгрузка. Т. е. груз выгружается у получателя.
                'HEADER_GOODS_UNLOADING' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_GOODS_UNLOADING_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'IS_GOODS_UNLOADING' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_IS_GOODS_UNLOADING'),
                    'GROUP' => 'dellin_delivery_settings'
                ),
                'UNLOADING_TYPE' => array(
                    'TYPE' => 'DROPDOWN',
                    'DEFAULT' => 'NULL',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_UNLOADING_TYPE'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'NULL' => GetMessage('DELLIN_SETTINGS_VALUES_LOADING_TYPE_BACK'),
                        '0xb83b7589658a3851440a853325d1bf69' => GetMessage('DELLIN_SETTINGS_VALUES_LOADING_TYPE_SIDE'),
                        '0xabb9c63c596b08f94c3664c930e77778' => GetMessage('DELLIN_SETTINGS_VALUES_LOADING_TYPE_UP'),
                    ),
                ),
                'UNLOADING_TRANSPORT_REQUIREMENTS' => array(
                    'TYPE' => 'DROPDOWN',
                    'DEFAULT' => 'NULL',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_TRANSPORT_REQUIREMENTS'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'NULL' => GetMessage('DELLIN_SETTINGS_VALUES_TRANSPORT_REQUIREMENTS_NO'),
                        '0x9951e0ff97188f6b4b1b153dfde3cfec' => GetMessage('DELLIN_SETTINGS_VALUES_TRANSPORT_REQUIREMENTS_OPEN'),
                        '0x818e8ff1eda1abc349318a478659af08' => GetMessage('DELLIN_SETTINGS_VALUES_TRANSPORT_REQUIREMENTS_TENT')
                    ),
                ),
                'UNLOADING_ADDITIONAL_EQUIPMENT' => array(
                    'TYPE' => 'DROPDOWN',
                    'DEFAULT' => 'NULL',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_ADDITIONAL_EQUIPMENT'),
                    'GROUP' => 'dellin_delivery_settings',
                    'VALUES' => array(
                        'NULL' => GetMessage('DELLIN_SETTINGS_VALUES_ADDITIONAL_EQUIPMENT_NO'),
                        '0x92fce2284f000b0241dad7c2e88b1655' => GetMessage('DELLIN_SETTINGS_VALUES_ADDITIONAL_EQUIPMENT_TAIL_LIFT'),
                        '0x88f93a2c37f106d94ff9f7ada8efe886' => GetMessage('DELLIN_SETTINGS_VALUES_ADDITIONAL_EQUIPMENT_MANIPULATOR'),
                    ),
                ),

                // Упаковка
                'HEADER_PACKING_FOR_GOODS' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

                'PACKING_FOR_GOODS_BOX' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_BOX'),
                    'GROUP' => 'dellin_delivery_settings',
                ),

                'PACKING_FOR_GOODS_HARD' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_HARD'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_ADDITIONAL' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_ADDITIONAL'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_BUBBLE' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_BUBBLE'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_BAG' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_BAG'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_PALLET' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_PALLET'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_CAR_GLASS' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_CAR_GLASS'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_CAR_PARTS' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_CAR_PARTS'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_COMPLEX_PALLET' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_COMPLEX_PALLET'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'PACKING_FOR_GOODS_COMPLEX_HARD' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_PACKING_FOR_GOODS_COMPLEX_HARD'),
                    'GROUP' => 'dellin_delivery_settings',
                ),

                // Внутригородская доставка
                'HEADER_INTERCITY' => array(
                    'TYPE' => 'SECTION',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_INTERCITY_SECTION_HEADER'),
                    'GROUP' => 'dellin_delivery_settings',
                ),
                'INTERCITY_HIDE' => array(
                    'TYPE' => 'CHECKBOX',
                    'DEFAULT' => 'N',
                    'TITLE' => GetMessage('DELLIN_SETTINGS_TITLE_INTERCITY_HIDE'),
                    'GROUP' => 'dellin_delivery_settings'
                ),

            ),
        );
        /*$config = array(
            'LOGIN'     =>array('VALUE'=>$arSettings['LOGIN']),
            'PASSWORD'  => array('VALUE'=>$arSettings['PASSWORD'])
        );
        ApiCore::setApiKey($arSettings['API_KEY']);
        $arParams = array(
            "appkey"    => $arSettings['API_KEY'],
            "sessionID" => DellinAPI::GetDellinSessionId($config),
            "code"      => $arSettings['KLADR_CODE_DELIVERY_FROM'],
            "direction" => "derival"
        );
            */

        return $arConfig;
    }

    /**
     * Установка параметров обработчика.
     * @param array $arSettings
     * @return string
     */
    function SetSettings($arSettings)
    {
        $arSettings['MARKUP_ABSOLUTE'] = (float)str_replace(',', '.', $arSettings['MARKUP_ABSOLUTE']);
        $arSettings['KLADR_CODE_DELIVERY_FROM'] = preg_replace('/[^\'0-9\s]/', '', $arSettings['KLADR_CODE_DELIVERY_FROM']);
        $arSettings['API_KEY'] = trim($arSettings['API_KEY']);
        self::savePathToOrder($arSettings['PATH_TO_ORDER']);
        return serialize($arSettings);
    }

    /**
     * @param $path
     * заменяем в файле include.php путь к странице заказа
     */
    static function savePathToOrder($path){
        if(empty($path)) $path = "/personal/order/make/";
        $pathToOrder = self::getPathToOrder();
        $text = str_replace($pathToOrder['path'],$path,$pathToOrder['context']);
        $file = fopen(self::$moduleIncludePath, 'w');
        fwrite($file, $text);
        fclose($file);
    }

    static function getPathToOrder(){
        $file = fopen(self::$moduleIncludePath, 'r');
        $text = fread($file, filesize(self::$moduleIncludePath));
        fclose($file);
        $re = '~\$pathToOrder = \"(.+)\"~i';
        $str = $text;
        preg_match($re, $str,$matches);
        $pathToOrder = $matches[1];
        return array('path'=>$pathToOrder,'context'=>$text);
    }
    /**
     * Получение списка стран и их ОПФ из Апи
     * @return array
     */
    public function getOpf(){
        $getData = $_REQUEST;
        $arCountriesList = array();
        $arOpfList = array();
        if(isset($getData['API_KEY']) && strlen(trim($getData['API_KEY'])) > 0){
            $opfList = ApiCore::GetOpfList($getData['API_KEY'])['list'];
            if(isset($opfList)){
                $countries = ApiCore::GetCountries($getData['API_KEY']);
                foreach($countries as $country){
                    $arCountriesList[$country['countryUID']] = $country['country'];
                }
                asort($arCountriesList);
                foreach($opfList as $id => $opf){
                    $arOpfList[$opf['countryUID']][$opf['uid']] = $opf['name'];
                    asort( $arOpfList[$opf['countryUID']]);
                }

            }
        }
        return array('opf'=>$arOpfList,'countries'=>$arCountriesList);
    }

    /**
     * Запрос параметров обработчика.
     * @param string $strSettings
     * @return mixed
     */
    function GetSettings($strSettings)
    {
        $arSettings = unserialize($strSettings);
        $pathToOrder = self::getPathToOrder();
        $arSettings['PATH_TO_ORDER'] = $pathToOrder['path'];
        return $arSettings;
    }

    /**
     * Проверка соответствия профиля доставки заказу.
     * @param array $arOrder
     * @param array $arConfig
     * @return array
     */
    function Compability($arOrder, $arConfig)
    {
        $profile_list = array();
        $response = DellinAPI::Calculate($arOrder, $arConfig);
        if ($response['STATUS'] == 'OK') {
            $profile_list[] = 'dellin_delivery_default';
        }
        return $profile_list;
    }


    /**
     * Получаем настройки доставки
     */
    public static function GetDeliveryConfig($deliveryId){
        if($deliveryId){
            $delivery = DeliveryManager::GetByID($deliveryId);
            $config = unserialize(unserialize($delivery['CONFIG']['MAIN']['OLD_SETTINGS']));
            $arNewConfig = array();
            foreach($config as $index => $configValue){
                $arNewConfig[$index] = array('VALUE'=>$configValue);
            }
            if($arNewConfig){
                return $arNewConfig;
            } else {
                return false;
            }
        }else{
            return false;
        }
    }


    /**
     * Рассчитываем итоговую стоимость и время доставки
     * @param $profile
     * @param array $arConfig
     * @param array $arOrder
     * @param $STEP
     * @param bool $TEMP
     * @return array
     */
    function Calculate($profile, $arConfig, $arOrder, $STEP, $TCEMP = false)
    {
        $response = DellinAPI::Calculate($arOrder, $arConfig);
        if(isset($response)){
            if ($response['STATUS'] == 'OK') {
                return array(
                    'RESULT' => 'OK',
                    'VALUE' => $response['BODY'][0],
                    'TRANSIT' => $response['BODY'][1]
                );
            }else{
                return array(
                    'RESULT' => 'ERROR',
                    'TEXT' => $response['BODY']
                );
            }
        }

    }

    /**
     * Получение статуса заказа из API
     * */
    function GetOrderStatus($orderId){
        if(CModule::IncludeModule('sale')){
            $order = Order::load($orderId);
            $deliveryCollection = $order->getShipmentCollection();
            $deliveryId = DeliveryManager::GetByID($deliveryCollection->current()->getField('DELIVERY_ID'))['PARENT_ID'];
            $config = self::GetDeliveryConfig($deliveryId);
            $params = array('appkey'=>$config['API_KEY']['VALUE'],'docid'=>$deliveryCollection->current()->getField('TRACKING_NUMBER'));
            if(isset($params['docid'])){
                $result = ApiCore::GetStatus($params);
                return $result;
            }else{
                return false;
            }

            if($result && !isset($result->errors)){
                return get_object_vars($result);
            }else{
                return false;
            }
        }else{
            echo GetMessage('DELLIN_MODULE_SALE_NOT_INCLUDED');
        }
    }

    protected static function GetDeliveryOrderData($orderId,$deliveryType){
        // собираем данные для заявки
        $arOrder = self::GetOrder($orderId);
        $config = self::GetDeliveryConfig($arOrder['CURRENT_DELIVERY_ID']);
        $sessionId = DellinAPI::GetDellinSessionId($config);
        $db_props = CSaleOrderPropsValue::GetOrderProps($orderId);
        $opfList = ApiCore::GetOpfList($config['API_KEY']['VALUE'])['list'];
        foreach($opfList as $opf){
          $arOpfList[$opf['uid']] = $opf;
        }
        while($arProps = $db_props->Fetch()){
            $arOrderProps[$arProps['CODE']] = $arProps;
        }
        $arOrder['QUANTITY'] = 0;
        $arOrder['FREIGHT_NAME']="";// названия товаров в заказе одной строкой

        //количество мест для заказа исходя из типа группировки по местам
        switch ($config['LOADING_GROUPING_OF_GOODS']['VALUE']) {
            case 'ONE_CARGO_SPACE':
                $arOrder['QUANTITY'] = 1;
                break;
            case 'SEPARATED_CARGO_SPACE':
                $arOrder['QUANTITY'] = count($arOrder['ITEMS']);
                break;
            case 'SINGLE_ITEM_SINGLE_SPACE':
                foreach($arOrder['ITEMS'] as $index=>$item){
                    $arOrder['QUANTITY'] += $item['QUANTITY'];
                }
                break;
        }
        //перечисление товаров для freight name
        foreach($arOrder['ITEMS'] as $index=>$item) {
            $arOrder['FREIGHT_NAME'] .= (($index != 0) ? "," : "") . $item['NAME'];
        }

        $arVolume = DellinAPI::CalculateVolume($arOrder,$config);
        $arWeight = DellinAPI::CalculateWeight($arOrder,$config);
        if(!$config['DELIVERY_DELAY']['VALUE'] ||  $config['DELIVERY_DELAY']['VALUE'] == 0){
            $deliveryDelay = 1;
        }else{
            $deliveryDelay = $config['DELIVERY_DELAY']['VALUE'];
        }
        $produceData = array(
            "appKey"                => $config['API_KEY']['VALUE'],
            "produceDate"           => date("Y-m-d",mktime(0, 0, 0, date("m")  , date("d")+$deliveryDelay, date("Y"))), //ориентировочная минимальная дата (сегодня + дней из настроек)
            "sizedVolume"           => $arVolume['sized'],
            "sizedWeight"           => $arWeight['sized'],
            "derivalPoint"          => $config['KLADR_CODE_DELIVERY_FROM']['VALUE'],
            "derivalDoor"           => ($config['IS_GOODS_LOADING']['VALUE'] == "Y")?true:false,
            "deliveryType"          => "1",
            "derivalPeriodVisit"    => array(
                "workStart" => ($config['WORK_START']['VALUE'] < 10?"0":"").$config['WORK_START']['VALUE'].":".(($config['WORK_START']['VALUE'] ==24)?"59":"00"),
                "workEnd"   => ($config['WORK_END']['VALUE'] <   10?"0":"").$config['WORK_END']['VALUE'].":".(($config['WORK_END']['VALUE'] ==24)?"59":"00")
            ),
            "oversizedWeight" => $arWeight['oversized'],
            "oversizedVolume" => $arVolume['oversized'],
            "length"          => (ApiCore::convertMMtoM($arOrder['MAX_LENGTH'])>0)?round(ApiCore::convertMMtoM($arOrder['MAX_LENGTH']),3):0.01,
            "width"           => (ApiCore::convertMMtoM($arOrder['MAX_WIDTH'])>0)?round(ApiCore::convertMMtoM($arOrder['MAX_WIDTH']),3):0.01,
            "height"          => (ApiCore::convertMMtoM($arOrder['MAX_HEIGHT'])>0)?round(ApiCore::convertMMtoM($arOrder['MAX_HEIGHT']),3):0.01
        );

        $produceDate = self::GetProduceDate($produceData,$config);
        if(!isset($produceDate->errors)){
            if($produceDate->nearestDate){//если дата не подошла
                $nearestDate = $produceDate->nearestDate;
            }else{
                $nearestDate = date("Y-m-d",mktime(0, 0, 0, date("m")  , date("d")+$deliveryDelay, date("Y")));
            }
        }else{
            return $produceDate;
        }

        //коды ИП для разных стран
        $receiverPPCodes = array(
            "0000028023" => "0xab91feea04f6d4ad48df42161b6c2e7a"
        );
        $location = CSaleLocation::GetByID($arOrder['LOCATION_TO']['VALUE']);
        $country = CSaleLocation::GetByID($location['COUNTRY_ID']);
        $deliveryData = array(
            "appKey"    => $config['API_KEY']['VALUE'],
            "delivery"  => array(
                "deliveryType" => array(
                    "type"  => $deliveryType,
                    "payer" => "sender"
                ),
                /*"simpleShipping" => "false",*/
                "derival"   => array(
                    "produceDate" => $nearestDate,
                    "variant"     => ($config['IS_GOODS_LOADING']['VALUE'] == "Y")?"address":"terminal",
                    "payer"       => "sender",
                ),
                "arrival"   => array(
                    "variant"     => ($config['IS_GOODS_UNLOADING']['VALUE'] == "Y")?"address":"terminal",
                    "payer"       => "sender",
                ),
            ),
            "members" => array(
                "requester" => array(
                    "role"  => "sender",
                    "email" => $config['REQUEST_EMAIL']['VALUE'] //почта
                ),
                "sender" => array(
                    "counteragent"=> array(
                        "form" => $config['SENDER_FORM']['VALUE'],
                        "name" => $config['SENDER_NAME']['VALUE'],
                        "inn"  => $config['SENDER_INN']['VALUE'],
                        "juridicalAddress" => ($arOpfList[$config['SENDER_FORM']['VALUE']]['juridical'] == 1)?array(
                            "street"    => $config['SENDER_JURIDICAL_KLADR_STREET']['VALUE'],
                            "house"     => $config['SENDER_JURIDICAL_HOUSE']['VALUE'],
                        ):array(),
                        "isAnonym" => false
                    ),
                    "contactPersons" => array(
                        array(
                            "name" => $config['SENDER_CONTACT_PERSON_NAME']['VALUE'],
                            "save" => false
                        )
                    ),
                    "phoneNumbers" => array(
                        array(
                            "number" => preg_replace("/[^0-9]/", '', $config['SENDER_CONTACT_PHONE']['VALUE'])
                        )
                    ),
                    "email" =>  $config['SENDER_CONTACT_EMAIL']['VALUE']
                ),
                "receiver" => array(
                    "counteragent" => array(
                        "form" => $receiverPPCodes[$country['CODE']],//физическое лицо. Если юр. лицо, то ниже counteragent изменится на массив параметров для юр. лица
                    ),
                    "contactPersons" => array(
                        array(
                            "name" => $arOrderProps['CONTACT_PERSON']['VALUE']?$arOrderProps['CONTACT_PERSON']['VALUE']:$arOrderProps['FIO']['VALUE'],
                            "save" => false
                        )
                    ),
                    "phoneNumbers"=> array(
                        array(
                            "number" => $str = preg_replace("/[^0-9]/", '', $arOrderProps['PHONE']['VALUE'])
                        )
                    ),
                    "email" => $arOrder['USER_EMAIL']
                ),
            ),
            "cargo" => array(
                "quantity"    => $arOrder['QUANTITY'],
                "length"      => (ApiCore::convertMMtoM($arOrder['MAX_LENGTH']) > 0)?ApiCore::convertMMtoM($arOrder['MAX_LENGTH']):0.1,
                "width"       => (ApiCore::convertMMtoM($arOrder['MAX_WIDTH']) > 0)?ApiCore::convertMMtoM($arOrder['MAX_WIDTH']):0.1,
                "height"      => (ApiCore::convertMMtoM($arOrder['MAX_HEIGHT']) > 0)?ApiCore::convertMMtoM($arOrder['MAX_HEIGHT']):0.1,
                "weight"      => round($arWeight['sized'],3),
                "totalVolume" => round($arVolume['sized'],3),
                "totalWeight" => round($arWeight['sized'],3),
                "insurance" => array(
                    "statedValue" =>($config['IS_INSURANCE_GOODS_WITH_DECLARED_PRICE']['VALUE'] == 'Y')?($arOrder['PRICE'] - $arOrder['PRICE_DELIVERY']):0,
                    "payer"       => "sender",
                    "term"        => false
                ),
                "freightName" => $arOrder['FREIGHT_NAME']
            ),
            "payment"=> array(
                "type" => "noncash",
                "primaryPayer" => "sender"
            ),
            'productInfo'=>array(
                'type'=> 4,
                'productType'=> 5,
                'info'=>array(
                    array(
                        'param'=> 'module version',
                        'value'=> 'beta'
                    )
                )
            )
        );
        if($sessionId){
            $deliveryData["sessionID"] = $sessionId;
        }

        if($config['COUNTERAGENT']['VALUE'] !== "NULL" && !empty($config['COUNTERAGENT']['VALUE'])){
            $deliveryData['members']['requester']["uid"] = $config['COUNTERAGENT']['VALUE'];
        }
        if($config['SENDER_JURIDICAL_BUILDING']['VALUE']) $deliveryData['members']['sender']['counteragent']['juridicalAddress']['building'] = $config['SENDER_JURIDICAL_BUILDING']['VALUE'];
        if($config['SENDER_JURIDICAL_STRUCTURE']['VALUE']) $deliveryData['members']['sender']['counteragent']['juridicalAddress']['structure'] = $config['SENDER_JURIDICAL_STRUCTURE']['VALUE'];
        if($config['SENDER_JURIDICAL_FLAT']['VALUE']) $deliveryData['members']['sender']['counteragent']['juridicalAddress']['flat'] = $config['SENDER_JURIDICAL_FLAT']['VALUE'];
        if($arOrder['PERSON_TYPE_ID'] == 2){
            $deliveryData['members']['receiver']['counteragent'] =
                array(
                    "form" => $arOrderProps['JURIDICAL_OPF']['VALUE'],
                    "name" => $arOrderProps['COMPANY']['VALUE'],
                    "inn"  => $arOrderProps['INN']['VALUE'],
                    "juridicalAddress" => array(
                        "street"    => $arOrderProps['JURIDICAL_STREET_KLADR']['VALUE']?:"",
                        "house"     => $arOrderProps['JURIDICAL_HOUSE_NUM']['VALUE']?:"",
                    ),
                    "isAnonym" =>false
                );
                if($arOrderProps['JURIDICAL_BUILDING_NUM']['VALUE']) $deliveryData['members']['receiver']['counteragent']['juridicalAddress']["building"] = $arOrderProps['JURIDICAL_BUILDING_NUM']['VALUE'];
                if($arOrderProps['JURIDICAL_STRUCTURE_NUM']['VALUE']) $deliveryData['members']['receiver']['counteragent']['juridicalAddress']["structure"] = $arOrderProps['JURIDICAL_STRUCTURE_NUM']['VALUE'];
                if($arOrderProps['JURIDICAL_FLAT_NUM']['VALUE']) $deliveryData['members']['receiver']['counteragent']['juridicalAddress']["flat"] = $arOrderProps['JURIDICAL_FLAT_NUM']['VALUE'];
        }
        if($arOrder['PERSON_TYPE_ID'] == 1){
            $deliveryData['members']['receiver']['counteragent']['name'] = $arOrderProps['FIO']['VALUE'];
            $deliveryData['members']['receiver']['counteragent']['phone'] =  preg_replace("/[^0-9]/", '', $arOrderProps['PHONE']['VALUE']);
            $deliveryData['members']['receiver']['counteragent']['isAnonym'] = true;
            $deliveryData['members']['receiver']['dataForReceipt'] = array('send'=>false);
        }

        if($config['IS_GOODS_LOADING']['VALUE'] != "Y"){
            $deliveryData["delivery"]["derival"]["terminalID"] = $config['TERMINAL_ID']['VALUE'];
        }else{
            $deliveryData["delivery"]["derival"]["address"]['street'] = $config['LOADING_ADDRESS']['VALUE'];
            $deliveryData["delivery"]["derival"]["address"]['house'] = $config['LOADING_HOUSE_NUM']['VALUE'];
            $deliveryData["delivery"]["derival"]["time"] = array(//если указан адрес
                "worktimeStart" => (($config['WORK_START']['VALUE']<10)?"0":"").$config['WORK_START']['VALUE'].":00",
                "worktimeEnd"   => (($config['WORK_END']['VALUE']<10)?"0":"").$config['WORK_END']['VALUE'].":00",
            );
            if($config['WORK_BREAK_START']['VALUE']){
                $deliveryData["delivery"]["derival"]["time"]["breakStart"] = (($config['WORK_BREAK_START']['VALUE']<10)?"0":"").$config['WORK_BREAK_START']['VALUE'].":00";
            }
            if($config['WORK_BREAK_END']['VALUE']){
                $deliveryData["delivery"]["derival"]["time"]["breakEnd"] = (($config['WORK_BREAK_END']['VALUE']<10)?"0":"").$config['WORK_BREAK_END']['VALUE'].":00";
            }
            $requirements = array();
            if($config['LOADING_TYPE']['VALUE'] !== "NULL") $requirements[] = $config['LOADING_TYPE']['VALUE'];
            if($config['LOADING_TRANSPORT_REQUIREMENTS']['VALUE'] !== "NULL") $requirements[] = $config['LOADING_TRANSPORT_REQUIREMENTS']['VALUE'];
            if($config['LOADING_ADDITIONAL_EQUIPMENT']['VALUE'] !== "NULL") $requirements[] = $config['LOADING_ADDITIONAL_EQUIPMENT']['VALUE'];
            if(!empty($requirements)) $deliveryData["delivery"]["derival"]['requirements'] = $requirements;
        }
        if($config['IS_GOODS_UNLOADING']['VALUE']!== "Y"){
            $deliveryData["delivery"]["arrival"]["terminalID"] = $arOrderProps['TERMINAL_ID']['VALUE'];
        }else{
            if($arOrderProps['STREET_KLADR']['VALUE']){
                $deliveryData["delivery"]["arrival"]["address"]['street'] = $arOrderProps['STREET_KLADR']['VALUE'];
            }else{
                $deliveryData["delivery"]["arrival"]["address"]['customStreet'] = array('code'=>$arOrder['ALT_LOCATION_KLADR'],'street'=>$arOrderProps['STREET']['VALUE']?:"Улица не указана");
            }
            $deliveryData["delivery"]["arrival"]["address"]['house'] = $arOrderProps['HOUSE_NUM']['VALUE'];
            $deliveryData["delivery"]["arrival"]["time"] = array(//если указан адрес
                "worktimeStart" => $arOrderProps['DELLIN_DELIVERYTIME_START']['VALUE'],
                "worktimeEnd"   => $arOrderProps['DELLIN_DELIVERYTIME_END']['VALUE']
            );
            $requirements = array();
            if($config['UNLOADING_TYPE']['VALUE'] !== "NULL") $requirements[] = $config['UNLOADING_TYPE']['VALUE'];
            if($config['UNLOADING_TRANSPORT_REQUIREMENTS']['VALUE'] !== "NULL") $requirements[] = $config['UNLOADING_TRANSPORT_REQUIREMENTS']['VALUE'];
            if($config['UNLOADING_ADDITIONAL_EQUIPMENT']['VALUE'] !== "NULL") $requirements[] = $config['UNLOADING_ADDITIONAL_EQUIPMENT']['VALUE'];
            if(!empty($requirements)) $deliveryData["delivery"]["arrival"]['requirements'] = $requirements;
        }

        /*if($config['PACKING_FOR_GOODS_HARD']['VALUE'] !== "NULL") $packages[] = array('uid'=>$config['PACKING_FOR_GOODS_HARD']['VALUE'],'payer'=>'sender');
        if($config['PACKING_FOR_GOODS_ADDITIONAL']['VALUE'] !== "NULL") $packages[] = array('uid'=>$config['PACKING_FOR_GOODS_ADDITIONAL']['VALUE'],'payer'=>'sender');
        if($config['PACKING_FOR_GOODS_BUBBLE']['VALUE'] !== "NULL") $packages[] =  array('uid'=>$config['PACKING_FOR_GOODS_BUBBLE']['VALUE'],'payer'=>'sender');
        if($config['PACKING_FOR_GOODS_BAG']['VALUE'] !== "NULL") $packages[] = array('uid'=>$config['PACKING_FOR_GOODS_BAG']['VALUE'],'payer'=>'sender');
        if($config['PACKING_FOR_GOODS_PALLET']['VALUE'] !== "NULL") $packages[] = array('uid'=>$config['PACKING_FOR_GOODS_PALLET']['VALUE'],'payer'=>'sender');*/
        $arPackages = DellinAPI::GetSelectedPackingTypesId('request',
            array(
                'box'=> $config['PACKING_FOR_GOODS_BOX']['VALUE'],
                'hard' => $config['PACKING_FOR_GOODS_HARD']['VALUE'],
                'additional' => $config['PACKING_FOR_GOODS_ADDITIONAL']['VALUE'],
                'bubble' => $config['PACKING_FOR_GOODS_BUBBLE']['VALUE'],
                'bag' => $config['PACKING_FOR_GOODS_BAG']['VALUE'],
                'pallet' => $config['PACKING_FOR_GOODS_PALLET']['VALUE'],
                'car_glass' =>$config['PACKING_FOR_GOODS_CAR_GLASS']['VALUE'],
                'car_parts' => $config['PACKING_FOR_GOODS_CAR_PARTS']['VALUE'],
                'complex_pallet' => $config['PACKING_FOR_GOODS_COMPLEX_PALLET']['VALUE'],
                'complex_hard' => $config['PACKING_FOR_GOODS_COMPLEX_HARD']['VALUE'],
            )
        );
        foreach($arPackages as $packageUid){
            $arPackage = array('uid'=>$packageUid,'payer'=>'sender');
            //если мешки или коробки, тоставим количество 1, т.к. поле обязательное, но мы не можем предсказать, сколько их нужно
            if($packageUid == "0x82750921BC8128924D74F982DD961379"
                || $packageUid=="0x947845D9BDC69EFA49630D8C080C4FBE"
                || $packageUid=="0xad97901b0ecef0f211e889fcf4624fed"
                || $packageUid=="0xad97901b0ecef0f211e889fcf4624fea"
            ) $arPackage['count']=1;
            $packages[] = $arPackage;
        }
        if(isset($packages)) $deliveryData["delivery"]["packages"] = $packages;
        return $deliveryData;

    }

    public function GetOrder($orderId){
        $arOrder = CSaleOrder::GetByID($orderId);
        $db_props = CSaleOrderPropsValue::GetOrderProps($orderId);
        $order = Order::load($orderId);
        $deliveryCollection = $order->getShipmentCollection();
        $arOrder['CURRENT_DELIVERY_ID'] = DeliveryManager::GetByID($deliveryCollection->current()->getField('DELIVERY_ID'))['PARENT_ID'];

        $altLocationPropId = CSaleOrderProps::GetList(array(),array('TYPE'=>'LOCATION','PERSON_TYPE_ID'=>$arOrder['PERSON_TYPE_ID']),false,false,array())->Fetch()['INPUT_FIELD_LOCATION'];
        while ($arProps = $db_props->Fetch()){
            if($arProps['CODE'] == "LOCATION") {
                $arOrder['LOCATION_TO'] = $arProps['VALUE'];
            }
            if($arProps['ORDER_PROPS_ID'] == $altLocationPropId) {
                $arOrder["ALT_LOCATION_NAME"] = $arProps['VALUE'];
            }
        }
        $arOrder['ALT_LOCATION_KLADR'] =   DellinAPI::GetCityKLADRCode($arOrder['LOCATION_TO'],$arOrder['ALT_LOCATION_NAME']);
        $dbBasketItems = CSaleBasket::GetList(
            array(),
            array(
                "ORDER_ID" => $orderId
            ),
            false,
            false,
            array()
        );
        if(CModule::IncludeModule('catalog')){
            $maxWeight = 0;
            $maxLength = 0;
            $maxWidth = 0;
            $maxHeight = 0;
            while($arItem = $dbBasketItems->Fetch()){
                $ar_res = CCatalogProduct::GetByID($arItem['PRODUCT_ID']);
                $arItem['DIMENSIONS'] = array(
                    "WIDTH"     => $ar_res['WIDTH'],
                    "HEIGHT"    => $ar_res['HEIGHT'],
                    "LENGTH"    => $ar_res['LENGTH'],
                );
                $arItem['WEIGHT'] = $ar_res['WEIGHT'];
                $arOrder['WEIGHT'] += $ar_res['WEIGHT']*$arItem['QUANTITY'];
                if($ar_res['WEIGHT']>$maxWeight) $maxWeight = $ar_res['WEIGHT'];
                if($ar_res['LENGTH']>$maxLength) $maxLength = $ar_res['LENGTH'];
                if($ar_res['WIDTH']>$maxWidth) $maxWidth = $ar_res['WIDTH'];
                if($ar_res['HEIGHT']>$maxHeight) $maxHeight = $ar_res['HEIGHT'];
                $arOrder['ITEMS'][] = $arItem;
            }
            $arOrder['MAX_WEIGHT'] = $maxWeight;
            $arOrder['MAX_LENGTH'] = $maxLength;
            $arOrder['MAX_WIDTH']  = $maxWidth;
            $arOrder['MAX_HEIGHT'] = $maxHeight;
        }
        return $arOrder;
    }

    /**
     * Пересчет заказа перед созданием заявки
     */
    function ReCalculateOrder($orderId){
        $arOrder = self::GetOrder($orderId);
        $config = self::GetDeliveryConfig($arOrder['CURRENT_DELIVERY_ID']);
        $calcResult = DellinAPI::Calculate($arOrder,$config);
        if($calcResult['STATUS'] !== "ERROR"){
            $price = $calcResult['BODY'][0];
            $period = $calcResult['BODY'][1];
            if((int)$arOrder['PRICE_DELIVERY'] !== (int) $price){
                $calcResult['PRICE_CHANGED'] = true;
            }
            if($calcResult['BODY'][3]){
                $calcResult['TERMINAL_RECEIVE_TIME_CHANGED'] = true;
            }
        }

        return $calcResult;

    }


    public function GetProduceDate($data,$deliveryConfig,$baseDate=false){
        if(!$baseDate) $baseDate = $data['produceDate'];
        $produceDate = ApiCore::sendApiRequest('produceDate',$data,'json');
        if(!isset($produceDate)){
            return false;
        }
        if(isset($produceDate->errors)){
            return $produceDate->errors;
        }
        if ($produceDate->produce_available) {//проверяем, возможна ли отгрузка хоть на какой-нибудь терминал в городе в эту дату
            if($deliveryConfig['IS_GOODS_LOADING']['VALUE'] == "N") {
                //если да, проверяем расписание терминала
                if(!empty($deliveryConfig['TERMINAL_ID']['VALUE'])){
                    $terminal = ApiCore::GetTerminals($deliveryConfig['API_KEY']['VALUE'],$deliveryConfig['KLADR_CODE_DELIVERY_FROM']['VALUE'], $deliveryConfig['TERMINAL_ID']['VALUE'])['terminal'];
                    foreach ($terminal->worktables->worktable as $worktable) {
                        $departmentname = trim($worktable->department);
                        if ($departmentname == "Приём и выдача груза" || $departmentname == "Приём груза") {
                            $weekDay = strtolower(date('l',strtotime($data['produceDate'])));
                            if ($worktable->{$weekDay} !== "-" ) {
                                if(str_replace(" ","",$worktable->{$weekDay}) == "24ч"){
                                    $startTime = "00:00";
                                    $endTime = "23:59";
                                    $workHours = array($startTime,$endTime);
                                }else{
                                    $workHours = explode("-", $worktable->{$weekDay});
                                    $startTime = strtotime(date('Y-m-d') . " " . $workHours[0]);
                                    $endTime = ($workHours[1] == '24:00')? strtotime(date('Y-m-d') . " 23:59"):strtotime(date('Y-m-d') . " " . $workHours[1]);
                                }

                                $shopStartTime = strtotime((($deliveryConfig['WORK_START']['VALUE']<10)?"0":"").$deliveryConfig['WORK_START']['VALUE'].":00");
                                $shopEndTime = strtotime((($deliveryConfig['WORK_END']['VALUE']<10)?"0":"").$deliveryConfig['WORK_END']['VALUE'].":00");
                                if ($shopEndTime <= $startTime || $shopStartTime >= $endTime) {
                                    $produceDate->changeTime = $workHours;
                                }

                                $produceDate->nearestDate = $data['produceDate'];
                                break;
                            }else{

                                $oldProduceDate = new DateTime($data['produceDate']);
                                $newProduceDate = $oldProduceDate->modify('+1 day');
                                $datediff = self::dateDifference($newProduceDate->format('Y-m-d'),$baseDate);
                                if($datediff <=2){
                                    $data['produceDate'] = $newProduceDate->format('Y-m-d');
                                    $produceDate = self::GetProduceDate($data,$deliveryConfig,$baseDate);
                                    $produceDate->DATE_CHANGED = true;
                                }
                            }
                        }
                    }
                    if(!isset($produceDate->nearestDate)){
                        $produceDate->produce_available = false;
                        $produceDate->errors['error_terminal_department'] = GetMessage('ERROR_TERMINAL_DEPARTMENT');
                    }
                }else{
                    $produceDate->errors['error_terminal_id'] = GetMessage('ERROR_TERMINAL_ID');
                }
            }else{
                $produceDate->nearestDate = $data['produceDate'];
            }
        }else{
            $oldProduceDate = new DateTime($data['produceDate']);
            $newProduceDate = $oldProduceDate->modify('+1 day');
            $datediff = self::dateDifference($newProduceDate->format('Y-m-d'),$baseDate);
            if($datediff <=2){
                $data['produceDate'] = $newProduceDate->format('Y-m-d');
                $produceDate = self::GetProduceDate($data,$deliveryConfig,$baseDate);
                $produceDate->DATE_CHANGED = true;
            }
            else{
                echo "<pre>";
                var_dump('ошибка: ближайшая доступная дата не найдена. Попробуйте изменить параметр "отложенная доставка" в настройках модуля');
                echo "</pre>";
                die();
            }
        }
        return $produceDate;
    }

    /**
     * Разница между датами
     * @param $date_1
     * @param $date_2
     * @param string $differenceFormat
     * @return string
     */
    protected function dateDifference($date_1 , $date_2 , $differenceFormat = '%a' )
    {
        $datetime1 = date_create($date_1);
        $datetime2 = date_create($date_2);

        $interval = date_diff($datetime1, $datetime2);

        return $interval->format($differenceFormat);

    }



    /**
     * Создание заявки на доставку и запись трекера в заказ Интернет-магазина
     *
    */
    function CreateOrder($orderId,$orderUpdated=false,$deliveryType){
        if(!$orderUpdated || $orderUpdated == "false"){
            $calcResult = self::ReCalculateOrder($orderId);
            if($calcResult['PRICE_CHANGED']){//если цена изменяется, сначала обновляем цену доставки в заказе
                $arFields = array(
                    "PRICE_DELIVERY"=>$calcResult['BODY'][0]
                );
                if(!CSaleOrder::Update($orderId, $arFields)){
                    $result = array("STATUS"=>"ERROR","BODY"=>GetMessage('DELLIN_ERROR_DELIVERY_PRICE'));
                }else{
                    $result = $calcResult;
                }
            }else{
                $result = $calcResult;
            }
        }else{
            $data = self::GetDeliveryOrderData($orderId,$deliveryType);
            if(isset($data->errors)){
                $errorBody = "Ошибки подбора даты отправки:<br/>";
                if(is_object($data->errors)) {
                    $arCategories = get_object_vars($data->errors);
                    foreach ($arCategories as $catCode => $errors) {
                        $arErrors = get_object_vars($errors);
                        foreach ($arErrors as $code => $error) {
                            $errorBody .= '[' . $code . '] ' . $error . ')<br/>';
                        }
                    }
                }else{
                    foreach ($data->errors as $code => $error) {
                        $errorBody .= '[' . $code . '] ' . $error . ')<br/>';
                    }
                }
                $result = array('STATUS' => 'ERROR', 'BODY' =>$errorBody);
            }else{

                $result = ApiCore::sendApiRequest('request',$data);
                if($result->data->state == 'success'){
                    $result = self::SaveTrackId($orderId,$result->data->requestID);
                }else{
                    $errorBody = "Ошибки создания заявки:<br/>";
                    foreach($result->errors as $error){
                        $errorBody .= '['.$error->code.'] '.$error->detail.'(поле:'.$error->fields[0].'<br/>';
                    }
                    $result = array('STATUS' => 'ERROR', 'BODY' =>$errorBody);
                }
            }
        }
        return $result;

    }


    /**
     * Сохранение кода отслеживания в заказ
    */
    function SaveTrackId($orderId, $trackId){
        $arFields = array(
            "TRACKING_NUMBER"=>$trackId
        );
        if($orderId = !CSaleOrder::Update($orderId, $arFields)){
            return array("STATUS"=>"ERROR","BODY"=>GetMessage('DELLIN_ERROR_TRACKING_NUMBER'));
        }else{
            return array("STATUS"=>"OK","ACTION"=>"CREATE", "BODY"=>array("TRACKING_NUMBER"=>$trackId));
        }
    }



}
